@@ -3,4 +3,4 @@
 dropdb $1
 createdb $1
 createlang plpgsql $1
-psql $1 </usr/share/pgsql/contrib/lwpostgis.sql
+psql $1 </usr/share/pgsql/contrib/postgis.sql
