@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: liblwgeom.h 4309 2009-07-16 21:32:24Z mcayland $
+ * $Id: liblwgeom.h 5168 2010-01-26 18:56:22Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -39,20 +39,27 @@
 #define INTEGRITY_CHECKS 1
 
 /*
- * Floating point comparitors.
- */
-#define PGIS_EPSILON 1e-12
-#define FP_MAX(A, B) ((A > B) ? A : B)
-#define FP_MIN(A, B) ((A < B) ? A : B)
-#define FP_LT(A, B) ((A + PGIS_EPSILON) < B)
-#define FP_LTEQ(A, B) ((A - PGIS_EPSILON) <= B)
-#define FP_GT(A, B) ((A - PGIS_EPSILON) > B)
-#define FP_GTEQ(A, B) ((A + PGIS_EPSILON) >= B)
+** Floating point comparitors.
+*/
+#define FP_TOLERANCE 1e-12
+#define FP_IS_ZERO(A) (fabs(A) <= FP_TOLERANCE)
+#define FP_MAX(A, B) (((A) > (B)) ? (A) : (B))
+#define FP_MIN(A, B) (((A) < (B)) ? (A) : (B))
+#define FP_EQUALS(A, B) (fabs((A)-(B)) <= FP_TOLERANCE)
+#define FP_NEQUALS(A, B) (fabs((A)-(B)) > FP_TOLERANCE)
+#define FP_LT(A, B) (((A) + FP_TOLERANCE) < (B))
+#define FP_LTEQ(A, B) (((A) - FP_TOLERANCE) <= (B))
+#define FP_GT(A, B) (((A) - FP_TOLERANCE) > (B))
+#define FP_GTEQ(A, B) (((A) + FP_TOLERANCE) >= (B))
 #define FP_CONTAINS_TOP(A, X, B) (FP_LT(A, X) && FP_LTEQ(X, B))
 #define FP_CONTAINS_BOTTOM(A, X, B) (FP_LTEQ(A, X) && FP_LT(X, B))
 #define FP_CONTAINS_INCL(A, X, B) (FP_LTEQ(A, X) && FP_LTEQ(X, B))
 #define FP_CONTAINS_EXCL(A, X, B) (FP_LT(A, X) && FP_LT(X, B))
 #define FP_CONTAINS(A, X, B) FP_CONTAINS_EXCL(A, X, B)
+
+/*
+** Utility true and false defines.
+*/
 #define LW_TRUE 1
 #define LW_FALSE 0
 
@@ -64,6 +71,10 @@
 #define NO_Z_VALUE NO_VALUE
 #define NO_M_VALUE NO_VALUE
 
+#ifndef MAXFLOAT
+#define MAXFLOAT      3.402823466e+38F
+#endif
+
 #ifndef C_H
 
 typedef unsigned int uint32;
@@ -240,6 +251,7 @@ typedef struct
 	double	f;	/* flattening     */
 	double	e;	/* eccentricity (first) */
 	double	e_sq;	/* eccentricity (first), squared */
+	double  radius; /* (2*a+b)/3 spherical average radius */
 	char	name[20]; /* name of ellipse */
 }
 SPHEROID;
@@ -527,7 +539,7 @@ extern void lwgeom_add_bbox(LWGEOM *lwgeom);
 extern void lwgeom_dropSRID(LWGEOM *lwgeom);
 
 /* Determine whether a LWGEOM can contain sub-geometries or not */
-extern int lwgeom_contains_subgeoms(int type);
+extern int lwgeom_is_collection(int type);
 
 /******************************************************************/
 
@@ -621,13 +633,14 @@ extern POINTARRAY *pointArray_construct(uchar *points, char hasz, char hasm,
 extern BOX3D *ptarray_compute_box3d(const POINTARRAY *pa);
 extern int ptarray_compute_box3d_p(const POINTARRAY *pa, BOX3D *out);
 
+
+
 /*
  * size of point represeneted in the POINTARRAY
  * 16 for 2d, 24 for 3d, 32 for 4d
  */
 extern int pointArray_ptsize(const POINTARRAY *pa);
 
-
 #define	POINTTYPE	1
 #define	LINETYPE	2
 #define	POLYGONTYPE	3
@@ -902,7 +915,7 @@ LWMSURFACE *lwmsurface_deserialize(uchar *serialized_form);
 
 LWGEOM *lwcollection_getsubgeom(LWCOLLECTION *col, int gnum);
 BOX3D *lwcollection_compute_box3d(LWCOLLECTION *col);
-
+LWCOLLECTION* lwcollection_extract(LWCOLLECTION *col, int type);
 
 /******************************************************************
  * SERIALIZED FORM functions
@@ -1171,10 +1184,21 @@ extern float nextafterf_custom(float x, float y);
 
 #define LW_MAX(a,b) ((a) >	(b) ? (a) : (b))
 #define LW_MIN(a,b) ((a) <= (b) ? (a) : (b))
-#define LW_ABS(a)   ((a) <	(0) ? (-a) : (a))
-
+#define LW_ABS(a)   ((a) <	(0) ? -(a) : (a))
 
+/* for the measure functions*/
+#define DIST2D_MAX		-1
+#define DIST2D_MIN		1
+ 
 /* general utilities */
+extern double distance2d_pt_pt(POINT2D *p1, POINT2D *p2);
+extern double distance2d_pt_seg(POINT2D *p, POINT2D *A, POINT2D *B);
+extern LWGEOM *lw_dist2d_distancepoint(uchar *lw1, uchar *lw2,int srid,int mode);
+extern LWGEOM *lw_dist2d_distanceline(uchar *lw1, uchar *lw2,int srid,int mode);
+extern double lwgeom_mindistance2d(uchar *lw1, uchar *lw2);
+extern double lwgeom_mindistance2d_tolerance(uchar *lw1, uchar *lw2, double tolerance);
+extern double lwgeom_maxdistance2d(uchar *lw1, uchar *lw2);
+extern double lwgeom_maxdistance2d_tolerance(uchar *lw1, uchar *lw2, double tolerance);
 extern double lwgeom_polygon_area(LWPOLY *poly);
 extern double lwgeom_polygon_perimeter(LWPOLY *poly);
 extern double lwgeom_polygon_perimeter2d(LWPOLY *poly);
@@ -1184,25 +1208,10 @@ extern void lwgeom_force2d_recursive(uchar *serialized, uchar *optr, size_t *ret
 extern void lwgeom_force3dz_recursive(uchar *serialized, uchar *optr, size_t *retsize);
 extern void lwgeom_force3dm_recursive(uchar *serialized, uchar *optr, size_t *retsize);
 extern void lwgeom_force4d_recursive(uchar *serialized, uchar *optr, size_t *retsize);
-extern double distance2d_pt_pt(POINT2D *p1, POINT2D *p2);
-extern double distance2d_pt_seg(POINT2D *p, POINT2D *A, POINT2D *B);
-extern double distance2d_seg_seg(POINT2D *A, POINT2D *B, POINT2D *C, POINT2D *D);
-extern double distance2d_pt_ptarray(POINT2D *p, POINTARRAY *pa);
-extern double distance2d_ptarray_ptarray(POINTARRAY *l1, POINTARRAY *l2);
 extern int pt_in_ring_2d(POINT2D *p, POINTARRAY *ring);
 extern int pt_in_poly_2d(POINT2D *p, LWPOLY *poly);
-extern double distance2d_ptarray_poly(POINTARRAY *pa, LWPOLY *poly);
-extern double distance2d_point_point(LWPOINT *point1, LWPOINT *point2);
-extern double distance2d_point_line(LWPOINT *point, LWLINE *line);
-extern double distance2d_line_line(LWLINE *line1, LWLINE *line2);
-extern double distance2d_point_poly(LWPOINT *point, LWPOLY *poly);
-extern double distance2d_poly_poly(LWPOLY *poly1, LWPOLY *poly2);
-extern double distance2d_line_poly(LWLINE *line, LWPOLY *poly);
 extern int azimuth_pt_pt(POINT2D *p1, POINT2D *p2, double *ret);
-extern double lwgeom_mindistance2d_recursive(uchar *lw1, uchar *lw2);
-extern double lwgeom_mindistance2d_recursive_tolerance(uchar *lw1, uchar *lw2, double tolerance);
 extern int lwgeom_pt_inside_circle(POINT2D *p, double cx, double cy, double rad);
-extern int32 lwgeom_npoints(uchar *serialized);
 extern char ptarray_isccw(const POINTARRAY *pa);
 extern void lwgeom_reverse(LWGEOM *lwgeom);
 extern void lwline_reverse(LWLINE *line);
@@ -1230,6 +1239,35 @@ extern int box2d_union_p(BOX2DFLOAT4 *b1, BOX2DFLOAT4 *b2, BOX2DFLOAT4 *ubox);
 extern int lwgeom_compute_box2d_p(LWGEOM *lwgeom, BOX2DFLOAT4 *box);
 void lwgeom_longitude_shift(LWGEOM *lwgeom);
 
+
+/**
+* @brief Check whether or not a lwgeom is big enough to warrant a bounding box.
+* 
+* Check whether or not a lwgeom is big enough to warrant a bounding box
+* when stored in the serialized form on disk. Currently only points are 
+* considered small enough to not require a bounding box, because the 
+* index operations can generate a large number of box-retrieval operations
+* when scanning keys.
+*/
+extern int lwgeom_needs_bbox(const LWGEOM *geom);
+
+/**
+* Count the total number of vertices in any #LWGEOM.
+*/
+extern int lwgeom_count_vertices(LWGEOM *geom);
+extern int32 lwgeom_npoints(uchar *serialized);
+
+/**
+* Return true of false depending on whether a geometry is an "empty" 
+* geometry (no vertices members)
+*/
+extern int lwgeom_is_empty(const LWGEOM *geom);
+
+/**
+* Return the dimensionality (relating to point/line/poly) of an lwgeom
+*/
+extern int lwgeom_dimensionality(LWGEOM *geom);
+
 /* Is lwgeom1 geometrically equal to lwgeom2 ? */
 char lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2);
 char ptarray_same(const POINTARRAY *pa1, const POINTARRAY *pa2);
@@ -1329,8 +1367,10 @@ extern POINTARRAY *ptarray_construct(char hasz, char hasm, unsigned int npoints)
 extern POINTARRAY *ptarray_addPoint(POINTARRAY *pa, uchar *p, size_t pdims,
 	                                    unsigned int where);
 extern POINTARRAY *ptarray_removePoint(POINTARRAY *pa, unsigned int where);
+extern POINTARRAY *ptarray_merge(POINTARRAY *pa1, POINTARRAY *pa2);
 
 extern int ptarray_isclosed2d(const POINTARRAY *pa);
+extern int ptarray_isclosed3d(const POINTARRAY *pa);
 
 extern void ptarray_longitude_shift(POINTARRAY *pa);
 
@@ -1339,6 +1379,8 @@ extern void ptarray_reverse(POINTARRAY *pa);
 extern POINTARRAY *ptarray_substring(POINTARRAY *, double, double);
 extern double ptarray_locate_point(POINTARRAY *, POINT2D *);
 extern void closest_point_on_segment(POINT2D *p, POINT2D *A, POINT2D *B, POINT2D *ret);
+extern LWLINE *lwline_measured_from_lwline(const LWLINE *lwline, double m_start, double m_end);
+extern LWMLINE* lwmline_measured_from_lwmline(const LWMLINE *lwmline, double m_start, double m_end);
 
 /*
  * Ensure every segment is at most 'dist' long.
