@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwgeom.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -14,7 +14,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#include "liblwgeom.h"
+#include "liblwgeom_internal.h"
 #include "wktparse.h"
 
 
@@ -227,6 +227,7 @@ BOX3D *lwgeom_compute_box3d(const LWGEOM *lwgeom)
 	return NULL;
 }
 
+
 int
 lwgeom_compute_box2d_p(LWGEOM *lwgeom, BOX2DFLOAT4 *buf)
 {
@@ -373,18 +374,18 @@ LWGEOM *lwpoint_as_lwgeom(LWPOINT *obj)
 ** Look-up for the correct MULTI* type promotion for singleton types.
 */
 static unsigned char MULTITYPE[16] =
-    {
-        0,
-        MULTIPOINTTYPE,
-        MULTILINETYPE,
-        MULTIPOLYGONTYPE,
-        0,0,0,0,
-        MULTICURVETYPE,
-        MULTICURVETYPE,
-        0,0,0,
-        MULTISURFACETYPE,
-        0,0
-    };
+{
+	0,
+	MULTIPOINTTYPE,
+	MULTILINETYPE,
+	MULTIPOLYGONTYPE,
+	0,0,0,0,
+	MULTICURVETYPE,
+	MULTICURVETYPE,
+	0,0,0,
+	MULTISURFACETYPE,
+	0,0
+};
 
 /**
 * Create a new LWGEOM of the appropriate MULTI* type.
@@ -403,7 +404,7 @@ lwgeom_as_multi(LWGEOM *lwgeom)
 	** This funx is a no-op only if a bbox cache is already present
 	** in input.
 	*/
-	if ( lwgeom_contains_subgeoms(TYPE_GETTYPE(lwgeom->type)) )
+	if ( lwgeom_is_collection(TYPE_GETTYPE(lwgeom->type)) )
 	{
 		return lwgeom_clone(lwgeom);
 	}
@@ -780,14 +781,14 @@ lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 	{
 		LWDEBUG(3, " type differ");
 
-		return 0;
+		return LW_FALSE;
 	}
 
 	if ( TYPE_GETZM(lwgeom1->type) != TYPE_GETZM(lwgeom2->type) )
 	{
 		LWDEBUG(3, " ZM flags differ");
 
-		return 0;
+		return LW_FALSE;
 	}
 
 	/* Check boxes if both already computed  */
@@ -798,7 +799,7 @@ lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 		{
 			LWDEBUG(3, " bounding boxes differ");
 
-			return 0;
+			return LW_FALSE;
 		}
 	}
 
@@ -823,7 +824,7 @@ lwgeom_same(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 	default:
 		lwerror("lwgeom_same: unsupported geometry type: %s",
 		        lwgeom_typename(TYPE_GETTYPE(lwgeom1->type)));
-		return 0;
+		return LW_FALSE;
 	}
 
 }
@@ -927,7 +928,7 @@ lwgeom_longitude_shift(LWGEOM *lwgeom)
 
 /** Return TRUE if the geometry may contain sub-geometries, i.e. it is a MULTI* or COMPOUNDCURVE */
 int
-lwgeom_contains_subgeoms(int type)
+lwgeom_is_collection(int type)
 {
 
 	switch (type)
@@ -976,4 +977,203 @@ void lwgeom_free(LWGEOM *lwgeom)
 	}
 	return;
 
-};
+}
+
+
+int lwgeom_needs_bbox(const LWGEOM *geom)
+{
+	assert(geom);
+	if ( TYPE_GETTYPE(geom->type) == POINTTYPE )
+	{
+		return LW_FALSE;
+	}
+	return LW_TRUE;
+}
+
+
+/*
+** Count points in an LWGEOM.
+*/
+
+static int lwcollection_count_vertices(LWCOLLECTION *col)
+{
+	int i = 0;
+	int v = 0; /* vertices */
+	assert(col);
+	for ( i = 0; i < col->ngeoms; i++ )
+	{
+		v += lwgeom_count_vertices(col->geoms[i]);
+	}
+	return v;
+}
+
+static int lwpolygon_count_vertices(LWPOLY *poly)
+{
+	int i = 0;
+	int v = 0; /* vertices */
+	assert(poly);
+	for ( i = 0; i < poly->nrings; i ++ )
+	{
+		v += poly->rings[i]->npoints;
+	}
+	return v;
+}
+
+static int lwline_count_vertices(LWLINE *line)
+{
+	assert(line);
+	if ( ! line->points )
+		return 0;
+	return line->points->npoints;
+}
+
+static int lwpoint_count_vertices(LWPOINT *point)
+{
+	assert(point);
+	if ( ! point->point )
+		return 0;
+	return 1;
+}
+
+int lwgeom_count_vertices(LWGEOM *geom)
+{
+	int result = 0;
+	LWDEBUGF(4, "got type %d", TYPE_GETTYPE(geom->type));
+	switch (TYPE_GETTYPE(geom->type))
+	{
+	case POINTTYPE:
+		result = lwpoint_count_vertices((LWPOINT *)geom);;
+		break;
+	case LINETYPE:
+		result = lwline_count_vertices((LWLINE *)geom);
+		break;
+	case POLYGONTYPE:
+		result = lwpolygon_count_vertices((LWPOLY *)geom);
+		break;
+	case MULTIPOINTTYPE:
+	case MULTILINETYPE:
+	case MULTIPOLYGONTYPE:
+	case COLLECTIONTYPE:
+		result = lwcollection_count_vertices((LWCOLLECTION *)geom);
+		break;
+	default:
+		lwerror("unsupported input geometry type: %d", TYPE_GETTYPE(geom->type));
+		break;
+	}
+	LWDEBUGF(3, "counted %d vertices", result);
+	return result;
+}
+
+static int lwpoint_is_empty(const LWPOINT *point)
+{
+	if ( ! point->point || point->point->npoints == 0 )
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+static int lwline_is_empty(const LWLINE *line)
+{
+	if ( !line->points || line->points->npoints == 0 )
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+static int lwpoly_is_empty(const LWPOLY *poly)
+{
+	if ( !poly->rings || poly->nrings == 0 )
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+static int lwcircstring_is_empty(const LWCIRCSTRING *circ)
+{
+	if ( !circ->points || circ->points->npoints == 0 )
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+static int lwcollection_is_empty(const LWCOLLECTION *col)
+{
+	if ( !col->geoms || col->ngeoms == 0 )
+		return LW_TRUE;
+	return LW_FALSE;
+}
+
+int lwgeom_is_empty(const LWGEOM *geom)
+{
+	int result = LW_FALSE;
+	LWDEBUGF(4, "got type %d", TYPE_GETTYPE(geom->type));
+	switch (TYPE_GETTYPE(geom->type))
+	{
+	case POINTTYPE:
+		return lwpoint_is_empty((LWPOINT*)geom);
+		break;
+	case LINETYPE:
+		return lwline_is_empty((LWLINE*)geom);
+		break;
+	case CIRCSTRINGTYPE:
+		return lwcircstring_is_empty((LWCIRCSTRING*)geom);
+		break;
+	case POLYGONTYPE:
+		return lwpoly_is_empty((LWPOLY*)geom);
+		break;
+	case MULTIPOINTTYPE:
+	case MULTILINETYPE:
+	case MULTIPOLYGONTYPE:
+	case COMPOUNDTYPE:
+	case MULTICURVETYPE:
+	case MULTISURFACETYPE:
+	case COLLECTIONTYPE:
+		return lwcollection_is_empty((LWCOLLECTION *)geom);
+		break;
+	default:
+		lwerror("unsupported input geometry type: %d", TYPE_GETTYPE(geom->type));
+		break;
+	}
+	return result;
+}
+
+static int lwcollection_dimensionality(LWCOLLECTION *col)
+{
+	int i;
+	int dimensionality = 0;
+	for ( i = 0; i < col->ngeoms; i++ )
+	{
+		int d = lwgeom_dimensionality(col->geoms[i]);
+		if ( d > dimensionality )
+			dimensionality = d;
+	}
+	return dimensionality;
+}
+
+extern int lwgeom_dimensionality(LWGEOM *geom)
+{
+	LWDEBUGF(4, "got type %d", TYPE_GETTYPE(geom->type));
+	switch (TYPE_GETTYPE(geom->type))
+	{
+	case POINTTYPE:
+	case MULTIPOINTTYPE:
+		return 0;
+		break;
+	case LINETYPE:
+	case CIRCSTRINGTYPE:
+	case MULTILINETYPE:
+	case COMPOUNDTYPE:
+	case MULTICURVETYPE:
+		return 1;
+		break;
+	case POLYGONTYPE:
+	case CURVEPOLYTYPE:
+	case MULTIPOLYGONTYPE:
+	case MULTISURFACETYPE:
+		return 2;
+		break;
+	case COLLECTIONTYPE:
+		return lwcollection_dimensionality((LWCOLLECTION *)geom);
+		break;
+	default:
+		lwerror("unsupported input geometry type: %d", TYPE_GETTYPE(geom->type));
+		break;
+	}
+	return 0;
+}
