@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: ptarray.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: ptarray.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -27,7 +27,11 @@ ptarray_construct(char hasz, char hasm, unsigned int npoints)
 	TYPE_SETZM(dims, hasz?1:0, hasm?1:0);
 	size = TYPE_NDIMS(dims)*npoints*sizeof(double);
 
-	ptlist = (uchar *)lwalloc(size);
+	if ( size )
+		ptlist = (uchar *)lwalloc(size);
+	else
+		ptlist = NULL;
+
 	pa = lwalloc(sizeof(POINTARRAY));
 	pa->dims = dims;
 	pa->serialized_pointlist = ptlist;
@@ -126,7 +130,7 @@ ptarray_compute_box2d(const POINTARRAY *pa)
 	result->ymin = pt.y;
 	result->ymax = pt.y;
 
-	for (t=1;t<pa->npoints;t++)
+	for (t=1; t<pa->npoints; t++)
 	{
 		getPoint2d_p(pa, t, &pt);
 		if (pt.x < result->xmin) result->xmin = pt.x;
@@ -300,6 +304,7 @@ ptarray_addPoint(POINTARRAY *pa, uchar *p, size_t pdims, unsigned int where)
 	return ret;
 }
 
+
 /**
  * @brief Remove a point from a pointarray.
  * 	@param which -  is the offset (starting at 0)
@@ -346,6 +351,41 @@ ptarray_removePoint(POINTARRAY *pa, unsigned int which)
 	return ret;
 }
 
+
+/**
+ * @brief Merge two given POINTARRAY and returns a pointer
+ * on the new aggregate one.
+ * Warning: this function free the two inputs POINTARRAY
+ * @return #POINTARRAY is newly allocated
+ */
+POINTARRAY *
+ptarray_merge(POINTARRAY *pa1, POINTARRAY *pa2)
+{
+	POINTARRAY *pa;
+	size_t ptsize = pointArray_ptsize(pa1);
+
+	if (TYPE_GETZM(pa1->dims) != TYPE_GETZM(pa2->dims))
+		lwerror("ptarray_cat: Mixed dimension");
+
+	pa = ptarray_construct( TYPE_HASZ(pa1->dims),
+	                        TYPE_HASM(pa1->dims),
+	                        pa1->npoints + pa2->npoints);
+
+	memcpy(         getPoint_internal(pa, 0),
+	                getPoint_internal(pa1, 0),
+	                ptsize*(pa1->npoints));
+
+	memcpy(         getPoint_internal(pa, pa1->npoints),
+	                getPoint_internal(pa2, 0),
+	                ptsize*(pa2->npoints));
+
+	lwfree(pa1);
+	lwfree(pa2);
+
+	return pa;
+}
+
+
 /**
  * @brief Clone a pointarray
  */
@@ -374,6 +414,13 @@ ptarray_isclosed2d(const POINTARRAY *in)
 	return 1;
 }
 
+int
+ptarray_isclosed3d(const POINTARRAY *in)
+{
+	if ( memcmp(getPoint_internal(in, 0), getPoint_internal(in, in->npoints-1), sizeof(POINT3D)) ) return 0;
+	return 1;
+}
+
 /**
  * @brief calculate the #BOX3D bounding box of a set of points
  * @return  a lwalloced #BOX3D, or NULL on empty array.
