@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_functions_basic.c 4168 2009-06-11 16:44:03Z pramsey $
+ * $Id: lwgeom_functions_basic.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -33,18 +33,23 @@ Datum LWGEOM_npoints(PG_FUNCTION_ARGS);
 Datum LWGEOM_nrings(PG_FUNCTION_ARGS);
 Datum LWGEOM_area_polygon(PG_FUNCTION_ARGS);
 Datum LWGEOM_dwithin(PG_FUNCTION_ARGS);
+Datum LWGEOM_dfullywithin(PG_FUNCTION_ARGS);
 Datum postgis_uses_stats(PG_FUNCTION_ARGS);
 Datum postgis_autocache_bbox(PG_FUNCTION_ARGS);
 Datum postgis_scripts_released(PG_FUNCTION_ARGS);
 Datum postgis_version(PG_FUNCTION_ARGS);
 Datum postgis_lib_version(PG_FUNCTION_ARGS);
+Datum postgis_libxml_version(PG_FUNCTION_ARGS);
 Datum postgis_lib_build_date(PG_FUNCTION_ARGS);
 Datum LWGEOM_length2d_linestring(PG_FUNCTION_ARGS);
 Datum LWGEOM_length_linestring(PG_FUNCTION_ARGS);
 Datum LWGEOM_perimeter2d_poly(PG_FUNCTION_ARGS);
 Datum LWGEOM_perimeter_poly(PG_FUNCTION_ARGS);
-Datum LWGEOM_mindistance2d(PG_FUNCTION_ARGS);
 Datum LWGEOM_maxdistance2d_linestring(PG_FUNCTION_ARGS);
+Datum LWGEOM_mindistance2d(PG_FUNCTION_ARGS);
+Datum LWGEOM_closestpoint(PG_FUNCTION_ARGS);
+Datum LWGEOM_shortestline2d(PG_FUNCTION_ARGS);
+Datum LWGEOM_longestline2d(PG_FUNCTION_ARGS);
 Datum LWGEOM_inside_circle_point(PG_FUNCTION_ARGS);
 Datum LWGEOM_collect(PG_FUNCTION_ARGS);
 Datum LWGEOM_accum(PG_FUNCTION_ARGS);
@@ -58,6 +63,8 @@ Datum LWGEOM_reverse(PG_FUNCTION_ARGS);
 Datum LWGEOM_forceRHR_poly(PG_FUNCTION_ARGS);
 Datum LWGEOM_noop(PG_FUNCTION_ARGS);
 Datum LWGEOM_zmflag(PG_FUNCTION_ARGS);
+Datum LWGEOM_hasz(PG_FUNCTION_ARGS);
+Datum LWGEOM_hasm(PG_FUNCTION_ARGS);
 Datum LWGEOM_ndims(PG_FUNCTION_ARGS);
 Datum LWGEOM_makepoint(PG_FUNCTION_ARGS);
 Datum LWGEOM_makepoint3dm(PG_FUNCTION_ARGS);
@@ -75,6 +82,8 @@ Datum LWGEOM_affine(PG_FUNCTION_ARGS);
 Datum LWGEOM_longitude_shift(PG_FUNCTION_ARGS);
 Datum optimistic_overlap(PG_FUNCTION_ARGS);
 Datum ST_GeoHash(PG_FUNCTION_ARGS);
+Datum ST_MakeEnvelope(PG_FUNCTION_ARGS);
+Datum ST_CollectionExtract(PG_FUNCTION_ARGS);
 
 void lwgeom_affine_ptarray(POINTARRAY *pa, double afac, double bfac, double cfac,
                            double dfac, double efac, double ffac, double gfac, double hfac, double ifac, double xoff, double yoff, double zoff);
@@ -189,6 +198,21 @@ Datum postgis_autocache_bbox(PG_FUNCTION_ARGS)
 }
 
 
+PG_FUNCTION_INFO_V1(postgis_libxml_version);
+Datum postgis_libxml_version(PG_FUNCTION_ARGS)
+{
+	char *ver = POSTGIS_LIBXML2_VERSION;
+	text *result;
+	result = lwalloc(VARHDRSZ  + strlen(ver));
+	SET_VARSIZE(result, VARHDRSZ + strlen(ver));
+	memcpy(VARDATA(result), ver, strlen(ver));
+	PG_RETURN_POINTER(result);
+}
+
+
+
+
+
 /**
  * Recursively count points in a SERIALIZED lwgeom
  */
@@ -1460,7 +1484,7 @@ Datum LWGEOM_force_collection(PG_FUNCTION_ARGS)
 	lwgeom = lwgeom_deserialize(SERIALIZED_FORM(geom));
 
 	/* alread a multi*, just make it a collection */
-	if ( lwgeom_contains_subgeoms(TYPE_GETTYPE(lwgeom->type)) )
+	if ( lwgeom_is_collection(TYPE_GETTYPE(lwgeom->type)) )
 	{
 		TYPE_SETTYPE(lwgeom->type, COLLECTIONTYPE);
 	}
@@ -1503,7 +1527,7 @@ Datum LWGEOM_force_multi(PG_FUNCTION_ARGS)
 	** in input. If bbox cache is not there we'll need to handle
 	** automatic bbox addition FOR_COMPLEX_GEOMS.
 	*/
-	if ( lwgeom_contains_subgeoms(TYPE_GETTYPE(geom->type)) && TYPE_HASBBOX(geom->type) )
+	if ( lwgeom_is_collection(TYPE_GETTYPE(geom->type)) && TYPE_HASBBOX(geom->type) )
 	{
 		PG_RETURN_POINTER(geom);
 	}
@@ -1512,8 +1536,6 @@ Datum LWGEOM_force_multi(PG_FUNCTION_ARGS)
 	/* deserialize into lwgeoms[0] */
 	lwgeom = lwgeom_deserialize(SERIALIZED_FORM(geom));
 	ogeom = lwgeom_as_multi(lwgeom);
-	printf("ogeom %p\n",ogeom);
-	printf("ogeom->type %d\n", ogeom->type);
 
 	result = pglwgeom_serialize(ogeom);
 
@@ -1522,7 +1544,110 @@ Datum LWGEOM_force_multi(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
-/* Minimum 2d distance between objects in geom1 and geom2. */
+/**
+Returns the point in first input geometry that is closest to the second input geometry
+*/
+
+PG_FUNCTION_INFO_V1(LWGEOM_closestpoint);
+Datum LWGEOM_closestpoint(PG_FUNCTION_ARGS)
+{
+
+
+	int srid;
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+
+	PG_LWGEOM *result=NULL;
+	LWGEOM *point;
+
+	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+
+	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
+	{
+		elog(ERROR,"Operation on two GEOMETRIES with different SRIDs\n");
+		PG_RETURN_NULL();
+	}
+
+	srid = pglwgeom_getSRID(geom1);
+
+	point = lw_dist2d_distancepoint( SERIALIZED_FORM(geom1),SERIALIZED_FORM(geom2), srid, DIST2D_MIN);
+	if (lwgeom_is_empty(point))
+	{
+		PG_RETURN_NULL();
+	}
+	result = pglwgeom_serialize(point);
+	PG_RETURN_POINTER(result);
+}
+
+/**
+Returns the shortest line between two geometries
+*/
+PG_FUNCTION_INFO_V1(LWGEOM_shortestline2d);
+Datum LWGEOM_shortestline2d(PG_FUNCTION_ARGS)
+{
+	int srid;
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+
+	PG_LWGEOM *result=NULL;
+	LWGEOM *theline;
+
+	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+
+	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
+	{
+		elog(ERROR,"Operation on two GEOMETRIES with different SRIDs\n");
+		PG_RETURN_NULL();
+	}
+
+	srid = pglwgeom_getSRID(geom1);
+
+	theline = lw_dist2d_distanceline( SERIALIZED_FORM(geom1),SERIALIZED_FORM(geom2), srid, DIST2D_MIN);
+	if (lwgeom_is_empty(theline))
+	{
+		PG_RETURN_NULL();
+	}
+	result = pglwgeom_serialize(theline);
+	PG_RETURN_POINTER(result);
+}
+
+/**
+Returns the longest line between two geometries
+*/
+PG_FUNCTION_INFO_V1(LWGEOM_longestline2d);
+Datum LWGEOM_longestline2d(PG_FUNCTION_ARGS)
+{
+	int srid;
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+
+	PG_LWGEOM *result=NULL;
+	LWGEOM *theline;
+
+	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+
+	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
+	{
+		elog(ERROR,"Operation on two GEOMETRIES with different SRIDs\n");
+		PG_RETURN_NULL();
+	}
+
+	srid = pglwgeom_getSRID(geom1);
+
+	theline = lw_dist2d_distanceline( SERIALIZED_FORM(geom1),SERIALIZED_FORM(geom2), srid, DIST2D_MAX);
+	if (lwgeom_is_empty(theline))
+	{
+		PG_RETURN_NULL();
+	}
+	result = pglwgeom_serialize(theline);
+	PG_RETURN_POINTER(result);
+}
+/**
+ Minimum 2d distance between objects in geom1 and geom2.
+ */
 PG_FUNCTION_INFO_V1(LWGEOM_mindistance2d);
 Datum LWGEOM_mindistance2d(PG_FUNCTION_ARGS)
 {
@@ -1541,19 +1666,27 @@ Datum LWGEOM_mindistance2d(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	mindist = lwgeom_mindistance2d_recursive(SERIALIZED_FORM(geom1),
-	          SERIALIZED_FORM(geom2));
+	mindist = lwgeom_mindistance2d(SERIALIZED_FORM(geom1),
+	                               SERIALIZED_FORM(geom2));
 
 	PROFSTOP(PROF_QRUN);
 	PROFREPORT("dist",geom1, geom2, NULL);
 
 	PG_FREE_IF_COPY(geom1, 0);
 	PG_FREE_IF_COPY(geom2, 1);
-
-	PG_RETURN_FLOAT8(mindist);
+	/*if called with empty geometries the ingoing mindistance is untouched, and makes us return NULL*/
+	if (mindist<MAXFLOAT)
+	{
+		PG_RETURN_FLOAT8(mindist);
+	}
+	PG_RETURN_NULL();
 }
 
-/* Minimum 2d distance between objects in geom1 and geom2. */
+/**
+Returns boolean describing if
+mininimum 2d distance between objects in
+geom1 and geom2 is shorter than tolerance
+*/
 PG_FUNCTION_INFO_V1(LWGEOM_dwithin);
 Datum LWGEOM_dwithin(PG_FUNCTION_ARGS)
 {
@@ -1579,7 +1712,7 @@ Datum LWGEOM_dwithin(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	mindist = lwgeom_mindistance2d_recursive_tolerance(
+	mindist = lwgeom_mindistance2d_tolerance(
 	              SERIALIZED_FORM(geom1),
 	              SERIALIZED_FORM(geom2),
 	              tolerance
@@ -1590,36 +1723,72 @@ Datum LWGEOM_dwithin(PG_FUNCTION_ARGS)
 
 	PG_FREE_IF_COPY(geom1, 0);
 	PG_FREE_IF_COPY(geom2, 1);
-
+	/*empty geometries cases should be right handled since return from underlying
+	 functions should be MAXFLOAT which causes false as answer*/
 	PG_RETURN_BOOL(tolerance >= mindist);
 }
 
-/*
- *  Maximum 2d distance between linestrings.
- *  Returns NULL if given geoms are not linestrings.
- *  This is very bogus (or I'm missing its meaning)
+/**
+Returns boolean describing if
+maximum 2d distance between objects in
+geom1 and geom2 is shorter than tolerance
+*/
+PG_FUNCTION_INFO_V1(LWGEOM_dfullywithin);
+Datum LWGEOM_dfullywithin(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+	double maxdist, tolerance;
+
+	PROFSTART(PROF_QRUN);
+
+	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	tolerance = PG_GETARG_FLOAT8(2);
+
+	if ( tolerance < 0 )
+	{
+		elog(ERROR,"Tolerance cannot be less than zero\n");
+		PG_RETURN_NULL();
+	}
+
+	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
+	{
+		elog(ERROR,"Operation on two GEOMETRIES with different SRIDs\n");
+		PG_RETURN_NULL();
+	}
+	maxdist = lwgeom_maxdistance2d_tolerance(
+	              SERIALIZED_FORM(geom1),
+	              SERIALIZED_FORM(geom2),
+	              tolerance
+	          );
+	PROFSTOP(PROF_QRUN);
+	PROFREPORT("dist",geom1, geom2, NULL);
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+	/*If function is feed with empty geometries we should return false*/
+	if (maxdist>-1)
+	{
+		PG_RETURN_BOOL(tolerance >= maxdist);
+	}
+	PG_RETURN_BOOL(LW_FALSE);
+}
+
+/**
+ Maximum 2d distance between objects in geom1 and geom2.
  */
 PG_FUNCTION_INFO_V1(LWGEOM_maxdistance2d_linestring);
 Datum LWGEOM_maxdistance2d_linestring(PG_FUNCTION_ARGS)
 {
-
 	PG_LWGEOM *geom1;
 	PG_LWGEOM *geom2;
-	LWLINE *line1;
-	LWLINE *line2;
-	double maxdist = 0;
-	int i;
+	double maxdist;
 
-	elog(ERROR, "This function is unimplemented yet");
-	PG_RETURN_NULL();
+	PROFSTART(PROF_QRUN);
 
 	geom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
-	line1 = lwline_deserialize(SERIALIZED_FORM(geom1));
-	if ( line1 == NULL ) PG_RETURN_NULL(); /* not a linestring */
-
 	geom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
-	line2 = lwline_deserialize(SERIALIZED_FORM(geom2));
-	if ( line2 == NULL ) PG_RETURN_NULL(); /* not a linestring */
 
 	if (pglwgeom_getSRID(geom1) != pglwgeom_getSRID(geom2))
 	{
@@ -1627,30 +1796,24 @@ Datum LWGEOM_maxdistance2d_linestring(PG_FUNCTION_ARGS)
 		PG_RETURN_NULL();
 	}
 
-	for (i=0; i<line1->points->npoints; i++)
-	{
-		POINT2D p;
-		double dist;
 
-		getPoint2d_p(line1->points, i, &p);
-		dist = distance2d_pt_ptarray(&p, line2->points);
+	maxdist = lwgeom_maxdistance2d(SERIALIZED_FORM(geom1),
+	                               SERIALIZED_FORM(geom2));
 
-		if (dist > maxdist) maxdist = dist;
-	}
+	PROFSTOP(PROF_QRUN);
+	PROFREPORT("maxdist",geom1, geom2, NULL);
 
 	PG_FREE_IF_COPY(geom1, 0);
 	PG_FREE_IF_COPY(geom2, 1);
-
-	PG_RETURN_FLOAT8(maxdist);
+	/*if called with empty geometries the ingoing mindistance is untouched, and makes us return NULL*/
+	if (maxdist>-1)
+	{
+		PG_RETURN_FLOAT8(maxdist);
+	}
+	PG_RETURN_NULL();
 }
 
-/**
- * @brief Longitude shift:
- *  	Y remains the same
- *  	X is converted:
- *	 		from -180..180 to 0..360
- *	 		from 0..360 to -180..180
- */
+
 PG_FUNCTION_INFO_V1(LWGEOM_longitude_shift);
 Datum LWGEOM_longitude_shift(PG_FUNCTION_ARGS)
 {
@@ -1880,10 +2043,8 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		SET_VARSIZE(result, nbytes);
 		result->ndim = 1;
 		result->elemtype = oid;
-
-#if POSTGIS_PGSQL_VERSION > 81
 		result->dataoffset = 0;
-#endif
+
 		memcpy(ARR_DIMS(result), &nelems, sizeof(int));
 		memcpy(ARR_LBOUND(result), &lbs, sizeof(int));
 		memcpy(ARR_DATA_PTR(result), geom, VARSIZE(geom));
@@ -1908,7 +2069,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		POSTGIS_DEBUGF(3, " %d bytes allocated for array", (int)nbytes);
 
 		POSTGIS_DEBUGF(3, " array start  @ %p", (void*)result);
-		POSTGIS_DEBUGF(3, " ARR_DATA_PTR @ %p (%d)",
+		POSTGIS_DEBUGF(3, " ARR_DATA_PTR @ %p (%ld)",
 		               ARR_DATA_PTR(result), (uchar *)ARR_DATA_PTR(result)-(uchar *)result);
 		POSTGIS_DEBUGF(3, " next element @ %p", (uchar *)result+oldsize);
 
@@ -1946,10 +2107,12 @@ Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS)
 	PG_LWGEOM *result=NULL;
 	LWGEOM **lwgeoms, *outlwg;
 	unsigned int outtype;
-	int i;
+	int i, count;
 	int SRID=-1;
 	size_t offset;
 	BOX2DFLOAT4 *box=NULL;
+	bits8 *bitmap;
+	int bitmask;
 
 	POSTGIS_DEBUG(2, "LWGEOM_collect_garray called.");
 
@@ -1987,89 +2150,115 @@ Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS)
 	 * array. Check input types to form output type.
 	 */
 	lwgeoms = palloc(sizeof(LWGEOM *)*nelems);
+	count = 0;
 	outtype = 0;
 	offset = 0;
+	bitmap = ARR_NULLBITMAP(array);
+	bitmask = 1;
 	for (i=0; i<nelems; i++)
 	{
-		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
-		unsigned int intype = TYPE_GETTYPE(geom->type);
-
-		offset += INTALIGN(VARSIZE(geom));
-
-		lwgeoms[i] = lwgeom_deserialize(SERIALIZED_FORM(geom));
-
-		POSTGIS_DEBUGF(3, "LWGEOM_collect_garray: geom %d deserialized", i);
-
-		if ( ! i )
+		/* Don't do anything for NULL values */
+		if ((bitmap && (*bitmap & bitmask) != 0) || !bitmap)
 		{
-			/* Get first geometry SRID */
-			SRID = lwgeoms[i]->SRID;
+			PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
+			unsigned int intype = TYPE_GETTYPE(geom->type);
 
-			/* COMPUTE_BBOX WHEN_SIMPLE */
-			if ( lwgeoms[i]->bbox )
-			{
-				box = box2d_clone(lwgeoms[i]->bbox);
-			}
-		}
-		else
-		{
-			/* Check SRID homogeneity */
-			if ( lwgeoms[i]->SRID != SRID )
-			{
-				elog(ERROR,
-				     "Operation on mixed SRID geometries");
-				PG_RETURN_NULL();
-			}
+			offset += INTALIGN(VARSIZE(geom));
 
-			/* COMPUTE_BBOX WHEN_SIMPLE */
-			if ( box )
+			lwgeoms[count] = lwgeom_deserialize(SERIALIZED_FORM(geom));
+
+			POSTGIS_DEBUGF(3, "LWGEOM_collect_garray: geom %d deserialized", i);
+
+			if ( ! count )
 			{
-				if ( lwgeoms[i]->bbox )
+				/* Get first geometry SRID */
+				SRID = lwgeoms[count]->SRID;
+
+				/* COMPUTE_BBOX WHEN_SIMPLE */
+				if ( lwgeoms[count]->bbox )
 				{
-					box->xmin = LW_MIN(box->xmin, lwgeoms[i]->bbox->xmin);
-					box->ymin = LW_MIN(box->ymin, lwgeoms[i]->bbox->ymin);
-					box->xmax = LW_MAX(box->xmax, lwgeoms[i]->bbox->xmax);
-					box->ymax = LW_MAX(box->ymax, lwgeoms[i]->bbox->ymax);
-				}
-				else
-				{
-					pfree(box);
-					box = NULL;
+					box = box2d_clone(lwgeoms[count]->bbox);
 				}
 			}
+			else
+			{
+				/* Check SRID homogeneity */
+				if ( lwgeoms[count]->SRID != SRID )
+				{
+					elog(ERROR,
+					     "Operation on mixed SRID geometries");
+					PG_RETURN_NULL();
+				}
+
+				/* COMPUTE_BBOX WHEN_SIMPLE */
+				if ( box )
+				{
+					if ( lwgeoms[count]->bbox )
+					{
+						box->xmin = LW_MIN(box->xmin, lwgeoms[count]->bbox->xmin);
+						box->ymin = LW_MIN(box->ymin, lwgeoms[count]->bbox->ymin);
+						box->xmax = LW_MAX(box->xmax, lwgeoms[count]->bbox->xmax);
+						box->ymax = LW_MAX(box->ymax, lwgeoms[count]->bbox->ymax);
+					}
+					else
+					{
+						pfree(box);
+						box = NULL;
+					}
+				}
+			}
+
+			lwgeom_dropSRID(lwgeoms[count]);
+			lwgeom_drop_bbox(lwgeoms[count]);
+
+			/* Output type not initialized */
+			if ( ! outtype )
+			{
+				/* Input is single, make multi */
+				if ( intype < 4 ) outtype = intype+3;
+				/* Input is multi, make collection */
+				else outtype = COLLECTIONTYPE;
+			}
+
+			/* Input type not compatible with output */
+			/* make output type a collection */
+			else if ( outtype != COLLECTIONTYPE && intype != outtype-3 )
+			{
+				outtype = COLLECTIONTYPE;
+			}
+
+			/* Advance NULL bitmap */
+			if (bitmap)
+			{
+				bitmask <<= 1;
+				if (bitmask == 0x100)
+				{
+					bitmap++;
+					bitmask = 1;
+				}
+			}
+
+			count++;
 		}
-
-
-		lwgeom_dropSRID(lwgeoms[i]);
-		lwgeom_drop_bbox(lwgeoms[i]);
-
-		/* Output type not initialized */
-		if ( ! outtype )
-		{
-			/* Input is single, make multi */
-			if ( intype < 4 ) outtype = intype+3;
-			/* Input is multi, make collection */
-			else outtype = COLLECTIONTYPE;
-		}
-
-		/* Input type not compatible with output */
-		/* make output type a collection */
-		else if ( outtype != COLLECTIONTYPE && intype != outtype-3 )
-		{
-			outtype = COLLECTIONTYPE;
-		}
-
 	}
 
 	POSTGIS_DEBUGF(3, "LWGEOM_collect_garray: outtype = %d", outtype);
 
-	outlwg = (LWGEOM *)lwcollection_construct(
-	             outtype, SRID,
-	             box, nelems, lwgeoms);
+	/* If we have been passed a complete set of NULLs then return NULL */
+	if (!outtype)
+	{
+		PG_RETURN_NULL();
+	}
+	else
+	{
+		outlwg = (LWGEOM *)lwcollection_construct(
+		             outtype, SRID,
+		             box, count, lwgeoms);
 
-	result = pglwgeom_serialize(outlwg);
+		result = pglwgeom_serialize(outlwg);
 
-	PG_RETURN_POINTER(result);
+		PG_RETURN_POINTER(result);
+	}
 }
 
 /**
@@ -2130,6 +2319,9 @@ Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS)
 	size_t offset;
 	int SRID=-1;
 
+	bits8 *bitmap;
+	int bitmask;
+
 	POSTGIS_DEBUG(2, "LWGEOM_makeline_garray called.");
 
 	/* Get input datum */
@@ -2169,34 +2361,51 @@ Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS)
 	lwpoints = palloc(sizeof(LWGEOM *)*nelems);
 	npoints = 0;
 	offset = 0;
+	bitmap = ARR_NULLBITMAP(array);
+	bitmask = 1;
 	for (i=0; i<nelems; i++)
 	{
-		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
-		offset += INTALIGN(VARSIZE(geom));
-
-		if ( TYPE_GETTYPE(geom->type) != POINTTYPE ) continue;
-
-		lwpoints[npoints++] =
-		    lwpoint_deserialize(SERIALIZED_FORM(geom));
-
-		/* Check SRID homogeneity */
-		if ( npoints == 1 )
+		/* Don't do anything for NULL values */
+		if ((bitmap && (*bitmap & bitmask) != 0) || !bitmap)
 		{
-			/* Get first geometry SRID */
-			SRID = lwpoints[npoints-1]->SRID;
-		}
-		else
-		{
-			if ( lwpoints[npoints-1]->SRID != SRID )
+			PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
+			offset += INTALIGN(VARSIZE(geom));
+
+			if ( TYPE_GETTYPE(geom->type) != POINTTYPE ) continue;
+
+			lwpoints[npoints++] =
+			    lwpoint_deserialize(SERIALIZED_FORM(geom));
+
+			/* Check SRID homogeneity */
+			if ( npoints == 1 )
 			{
-				elog(ERROR,
-				     "Operation on mixed SRID geometries");
-				PG_RETURN_NULL();
+				/* Get first geometry SRID */
+				SRID = lwpoints[npoints-1]->SRID;
+			}
+			else
+			{
+				if ( lwpoints[npoints-1]->SRID != SRID )
+				{
+					elog(ERROR,
+					     "Operation on mixed SRID geometries");
+					PG_RETURN_NULL();
+				}
+			}
+
+			POSTGIS_DEBUGF(3, "LWGEOM_makeline_garray: element %d deserialized",
+			               i);
+		}
+
+		/* Advance NULL bitmap */
+		if (bitmap)
+		{
+			bitmask <<= 1;
+			if (bitmask == 0x100)
+			{
+				bitmap++;
+				bitmask = 1;
 			}
 		}
-
-		POSTGIS_DEBUGF(3, "LWGEOM_makeline_garray: element %d deserialized",
-		               i);
 	}
 
 	/* Return null on 0-points input array */
@@ -2327,7 +2536,7 @@ Datum LWGEOM_expand(PG_FUNCTION_ARGS)
 {
 	PG_LWGEOM *geom = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	double d = PG_GETARG_FLOAT8(1);
-	BOX2DFLOAT4 box;
+	BOX3D box3d;
 	POINT2D *pts = lwalloc(sizeof(POINT2D)*5);
 	POINTARRAY *pa[1];
 	LWPOLY *poly;
@@ -2337,7 +2546,7 @@ Datum LWGEOM_expand(PG_FUNCTION_ARGS)
 	POSTGIS_DEBUG(2, "LWGEOM_expand called.");
 
 	/* get geometry box  */
-	if ( ! getbox2d_p(SERIALIZED_FORM(geom), &box) )
+	if ( ! compute_serialized_box3d_p(SERIALIZED_FORM(geom), &box3d) )
 	{
 		/* must be an EMPTY geometry */
 		PG_RETURN_POINTER(geom);
@@ -2347,19 +2556,19 @@ Datum LWGEOM_expand(PG_FUNCTION_ARGS)
 	SRID = lwgeom_getsrid(SERIALIZED_FORM(geom));
 
 	/* expand it */
-	expand_box2d(&box, d);
+	expand_box3d(&box3d, d);
 
 	/* Assign coordinates to POINT2D array */
-	pts[0].x = box.xmin;
-	pts[0].y = box.ymin;
-	pts[1].x = box.xmin;
-	pts[1].y = box.ymax;
-	pts[2].x = box.xmax;
-	pts[2].y = box.ymax;
-	pts[3].x = box.xmax;
-	pts[3].y = box.ymin;
-	pts[4].x = box.xmin;
-	pts[4].y = box.ymin;
+	pts[0].x = box3d.xmin;
+	pts[0].y = box3d.ymin;
+	pts[1].x = box3d.xmin;
+	pts[1].y = box3d.ymax;
+	pts[2].x = box3d.xmax;
+	pts[2].y = box3d.ymax;
+	pts[3].x = box3d.xmax;
+	pts[3].y = box3d.ymin;
+	pts[4].x = box3d.xmin;
+	pts[4].y = box3d.ymin;
 
 	/* Construct point array */
 	pa[0] = lwalloc(sizeof(POINTARRAY));
@@ -2368,7 +2577,7 @@ Datum LWGEOM_expand(PG_FUNCTION_ARGS)
 	pa[0]->npoints = 5;
 
 	/* Construct polygon  */
-	poly = lwpoly_construct(SRID, box2d_clone(&box), 1, pa);
+	poly = lwpoly_construct(SRID, ptarray_compute_box2d(pa[0]), 1, pa);
 
 	/* Construct PG_LWGEOM  */
 	result = pglwgeom_serialize((LWGEOM *)poly);
@@ -2467,26 +2676,24 @@ Datum LWGEOM_envelope(PG_FUNCTION_ARGS)
 	{
 		LWPOLY *poly;
 		POINT2D *pts = lwalloc(sizeof(POINT2D)*5);
-		BOX2DFLOAT4 box2d;
-		getbox2d_p(SERIALIZED_FORM(geom), &box2d);
 
 		/* Assign coordinates to POINT2D array */
-		pts[0].x = box2d.xmin;
-		pts[0].y = box2d.ymin;
-		pts[1].x = box2d.xmin;
-		pts[1].y = box2d.ymax;
-		pts[2].x = box2d.xmax;
-		pts[2].y = box2d.ymax;
-		pts[3].x = box2d.xmax;
-		pts[3].y = box2d.ymin;
-		pts[4].x = box2d.xmin;
-		pts[4].y = box2d.ymin;
+		pts[0].x = box.xmin;
+		pts[0].y = box.ymin;
+		pts[1].x = box.xmin;
+		pts[1].y = box.ymax;
+		pts[2].x = box.xmax;
+		pts[2].y = box.ymax;
+		pts[3].x = box.xmax;
+		pts[3].y = box.ymin;
+		pts[4].x = box.xmin;
+		pts[4].y = box.ymin;
 
 		/* Construct point array */
 		pa = pointArray_construct((uchar *)pts, 0, 0, 5);
 
 		/* Construct polygon  */
-		poly = lwpoly_construct(SRID, box2d_clone(&box2d), 1, &pa);
+		poly = lwpoly_construct(SRID, ptarray_compute_box2d(pa), 1, &pa);
 
 		/* Serialize polygon */
 		ser = lwpoly_serialize(poly);
@@ -2640,6 +2847,21 @@ Datum LWGEOM_zmflag(PG_FUNCTION_ARGS)
 	PG_RETURN_INT16(ret);
 }
 
+PG_FUNCTION_INFO_V1(LWGEOM_hasz);
+Datum LWGEOM_hasz(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *in = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_RETURN_BOOL(TYPE_HASZ(in->type));
+}
+
+PG_FUNCTION_INFO_V1(LWGEOM_hasm);
+Datum LWGEOM_hasm(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *in = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_RETURN_BOOL(TYPE_HASM(in->type));
+}
+
+
 PG_FUNCTION_INFO_V1(LWGEOM_hasBBOX);
 Datum LWGEOM_hasBBOX(PG_FUNCTION_ARGS)
 {
@@ -2705,6 +2927,56 @@ Datum LWGEOM_same(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(result);
 }
 
+PG_FUNCTION_INFO_V1(ST_MakeEnvelope);
+Datum ST_MakeEnvelope(PG_FUNCTION_ARGS)
+{
+	LWPOLY *poly;
+	PG_LWGEOM *result;
+	POINTARRAY **pa;
+	double *pts;
+	double x1, y1, x2, y2;
+	int srid;
+
+	POSTGIS_DEBUG(2, "ST_MakeEnvelope called");
+
+	x1 = PG_GETARG_FLOAT8(0);
+	y1 = PG_GETARG_FLOAT8(1);
+	x2 = PG_GETARG_FLOAT8(2);
+	y2 = PG_GETARG_FLOAT8(3);
+	srid = PG_GETARG_INT32(4);
+
+	pa = (POINTARRAY**)palloc(sizeof(POINTARRAY*));
+	pa[0] = ptarray_construct(0, 0, 5);
+	pts = (double*)(pa[0]->serialized_pointlist);
+
+	/* 1st point */
+	pts[0] = x1;
+	pts[1] = y1;
+
+	/* 2nd point */
+	pts[2] = x1;
+	pts[3] = y2;
+
+	/* 3rd point */
+	pts[4] = x2;
+	pts[5] = y2;
+
+	/* 4th point */
+	pts[6] = x2;
+	pts[7] = y1;
+
+	/* 5th point */
+	pts[8] = x1;
+	pts[9] = y1;
+
+	poly = lwpoly_construct(srid, ptarray_compute_box2d(pa[0]), 1, pa);
+
+	result = pglwgeom_serialize((LWGEOM*)poly);
+	lwpoly_free(poly);
+
+	PG_RETURN_POINTER(result);
+}
+
 PG_FUNCTION_INFO_V1(LWGEOM_makepoint);
 Datum LWGEOM_makepoint(PG_FUNCTION_ARGS)
 {
@@ -3323,3 +3595,38 @@ Datum ST_GeoHash(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 
 }
+
+PG_FUNCTION_INFO_V1(ST_CollectionExtract);
+Datum ST_CollectionExtract(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *input = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *output;
+	LWGEOM *lwgeom = pglwgeom_deserialize(input);
+	LWCOLLECTION *lwcol = NULL;
+	int type = PG_GETARG_INT32(1);
+	int lwgeom_type = TYPE_GETTYPE(lwgeom->type);
+
+	/* Ensure the right type was input */
+	if ( ! ( type == POINTTYPE || type == LINETYPE || type == POLYGONTYPE ) )
+	{
+		lwgeom_free(lwgeom);
+		elog(ERROR, "ST_CollectionExtract: only point, linestring and polygon may be extracted");
+		PG_RETURN_NULL();
+	}
+
+	/* Mirror non-collections right back */
+	if ( ! lwgeom_is_collection(lwgeom_type) )
+	{
+		output = palloc(VARSIZE(input));
+		memcpy(VARDATA(output), VARDATA(input), VARSIZE(input) - VARHDRSZ);
+		SET_VARSIZE(output, VARSIZE(input));
+		lwgeom_free(lwgeom);
+		PG_RETURN_POINTER(output);
+	}
+
+	lwcol = lwcollection_extract((LWCOLLECTION*)lwgeom, type);
+	output = pglwgeom_serialize((LWGEOM*)lwcol);
+	lwgeom_free(lwgeom);
+
+	PG_RETURN_POINTER(output);
+}
