@@ -1,10 +1,12 @@
 /**********************************************************************
- * $Id: lwgeom_geos.c 4237 2009-07-01 15:42:01Z pramsey $
+ * $Id: lwgeom_geos.c 5181 2010-02-01 17:35:55Z pramsey $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
- * Copyright 2001-2003 Refractions Research Inc.
+ *
+ * Copyright 2009 Sandro Santilli <strk@keybit.net>
  * Copyright 2008 Paul Ramsey <pramsey@cleverelephant.ca>
+ * Copyright 2001-2003 Refractions Research Inc.
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU General Public Licence. See the COPYING file.
@@ -15,6 +17,8 @@
 #include "lwgeom_rtree.h"
 #include "lwgeom_geos_prepared.h"
 
+#include <string.h>
+
 
 /**
 ** NOTE: Buffer-based GeomUnion has been disabled due to
@@ -58,6 +62,8 @@ Datum LWGEOM_buildarea(PG_FUNCTION_ARGS); /* TODO: rename to match others
 */
 Datum linemerge(PG_FUNCTION_ARGS);
 Datum coveredby(PG_FUNCTION_ARGS);
+Datum hausdorffdistance(PG_FUNCTION_ARGS);
+Datum hausdorffdistancedensify(PG_FUNCTION_ARGS);
 
 Datum pgis_union_geometry_array_old(PG_FUNCTION_ARGS);
 Datum pgis_union_geometry_array(PG_FUNCTION_ARGS);
@@ -86,6 +92,97 @@ Datum postgis_geos_version(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 }
 
+
+#if POSTGIS_GEOS_VERSION >= 32
+
+/**
+ *  @brief Compute the Hausdorff distance thanks to the corresponding GEOS function
+ *  @example hausdorffdistance {@link #hausdorffdistance} - SELECT st_hausdorffdistance(
+ *      'POLYGON((0 0, 0 2, 1 2, 2 2, 2 0, 0 0))'::geometry,
+ *      'POLYGON((0.5 0.5, 0.5 2.5, 1.5 2.5, 2.5 2.5, 2.5 0.5, 0.5 0.5))'::geometry);
+ */
+
+PG_FUNCTION_INFO_V1(hausdorffdistance);
+Datum hausdorffdistance(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+	GEOSGeometry *g1;
+	GEOSGeometry *g2;
+	double result;
+	int retcode;
+
+	POSTGIS_DEBUG(2, "hausdorff_distance called");
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+
+	initGEOS(lwnotice, lwnotice);
+
+	g1 = (GEOSGeometry *)POSTGIS2GEOS(geom1);
+	g2 = (GEOSGeometry *)POSTGIS2GEOS(geom2);
+	retcode = GEOSHausdorffDistance(g1, g2, &result);
+	GEOSGeom_destroy(g1);
+	GEOSGeom_destroy(g2);
+
+	if (retcode == 0)
+	{
+		elog(ERROR,"GEOS HausdorffDistance() threw an error!");
+		PG_RETURN_NULL(); /*never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_FLOAT8(result);
+}
+
+/**
+ *  @brief Compute the Hausdorff distance with densification thanks to the corresponding GEOS function
+ *  @example hausdorffdistancedensify {@link #hausdorffdistancedensify} - SELECT st_hausdorffdistancedensify(
+ *      'POLYGON((0 0, 0 2, 1 2, 2 2, 2 0, 0 0))'::geometry,
+ *      'POLYGON((0.5 0.5, 0.5 2.5, 1.5 2.5, 2.5 2.5, 2.5 0.5, 0.5 0.5))'::geometry, 0.5);
+ */
+
+PG_FUNCTION_INFO_V1(hausdorffdistancedensify);
+Datum hausdorffdistancedensify(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *geom1;
+	PG_LWGEOM *geom2;
+	GEOSGeometry *g1;
+	GEOSGeometry *g2;
+	double densifyFrac;
+	double result;
+	int retcode;
+
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	densifyFrac = PG_GETARG_FLOAT8(2);
+
+	initGEOS(lwnotice, lwnotice);
+
+	g1 = (GEOSGeometry *)POSTGIS2GEOS(geom1);
+	g2 = (GEOSGeometry *)POSTGIS2GEOS(geom2);
+	retcode = GEOSHausdorffDistanceDensify(g1, g2, densifyFrac, &result);
+	GEOSGeom_destroy(g1);
+	GEOSGeom_destroy(g2);
+
+	if (retcode == 0)
+	{
+		elog(ERROR,"GEOS HausdorffDistanceDensify() threw an error!");
+		PG_RETURN_NULL(); /*never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_FLOAT8(result);
+}
+
+#endif
+
+
 /**
  * @brief This is the final function for GeomUnion
  * 			aggregate. Will have as input an array of Geometries.
@@ -112,10 +209,8 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 #if POSTGIS_DEBUG_LEVEL > 0
 	static int call=1;
 #endif
-#if POSTGIS_GEOS_VERSION >= 31
 	int gotsrid = 0;
 	int allpolys=1;
-#endif
 
 #if POSTGIS_DEBUG_LEVEL > 0
 	call++;
@@ -150,8 +245,6 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 	/* Ok, we really need geos now ;) */
 	initGEOS(lwnotice, lwnotice);
 
-#if POSTGIS_GEOS_VERSION >= 31
-
 	/*
 	** First, see if all our elements are POLYGON/MULTIPOLYGON
 	** If they are, we can use UnionCascaded for faster results.
@@ -187,7 +280,7 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 
 		/* Advance NULL bitmap */
 		if (bitmap)
-                {
+		{
 			bitmask <<= 1;
 			if (bitmask == 0x100)
 			{
@@ -291,7 +384,6 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 	}
 	else
 	{
-#endif /* POSTGIS_GEOS_VERSION >= 31 */
 		/*
 		** Heterogeneous result, let's slog through this one union at a time.
 		*/
@@ -305,14 +397,14 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 			{
 				PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
 				offset += INTALIGN(VARSIZE(geom));
-	
+
 				pgis_geom = geom;
-	
+
 				POSTGIS_DEBUGF(3, "geom %d @ %p", i, geom);
-	
+
 				/* Check is3d flag */
 				if ( TYPE_HASZ(geom->type) ) is3d = 1;
-	
+
 				/* Check SRID homogeneity and initialize geos result */
 				if ( ! geos_result )
 				{
@@ -323,12 +415,12 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 				else
 				{
 					errorIfSRIDMismatch(SRID, pglwgeom_getSRID(geom));
-	
+
 					g1 = POSTGIS2GEOS(pgis_geom);
-	
+
 					POSTGIS_DEBUGF(3, "unite_garray(%d): adding geom %d to union (%s)",
-						call, i, lwgeom_typename(TYPE_GETTYPE(geom->type)));
-	
+					               call, i, lwgeom_typename(TYPE_GETTYPE(geom->type)));
+
 					g2 = GEOSUnion(g1, geos_result);
 					if ( g2 == NULL )
 					{
@@ -352,7 +444,7 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 					bitmask = 1;
 				}
 			}
-			
+
 		}
 
 		/* If geos_result is set then we found at least one non-NULL geometry */
@@ -368,9 +460,7 @@ Datum pgis_union_geometry_array(PG_FUNCTION_ARGS)
 			PG_RETURN_NULL();
 		}
 
-#if POSTGIS_GEOS_VERSION >= 31
 	}
-#endif
 
 	if ( result == NULL )
 	{
@@ -690,8 +780,6 @@ Datum convexhull(PG_FUNCTION_ARGS)
 
 }
 
-#if POSTGIS_GEOS_VERSION >= 30
-
 PG_FUNCTION_INFO_V1(topologypreservesimplify);
 Datum topologypreservesimplify(PG_FUNCTION_ARGS)
 {
@@ -735,7 +823,6 @@ Datum topologypreservesimplify(PG_FUNCTION_ARGS)
 	PG_FREE_IF_COPY(geom1, 0);
 	PG_RETURN_POINTER(result);
 }
-#endif /* POSTGIS_GEOS_VERSION >= 30 */
 
 PG_FUNCTION_INFO_V1(buffer);
 Datum buffer(PG_FUNCTION_ARGS)
@@ -745,12 +832,36 @@ Datum buffer(PG_FUNCTION_ARGS)
 	GEOSGeometry *g1, *g3;
 	PG_LWGEOM *result;
 	int quadsegs = 8; /* the default */
+	int nargs;
+	enum
+	{
+		ENDCAP_ROUND = 1,
+		ENDCAP_FLAT = 2,
+		ENDCAP_SQUARE = 3
+	};
+	enum
+	{
+		JOIN_ROUND = 1,
+		JOIN_MITRE = 2,
+		JOIN_BEVEL = 3
+	};
+	static const double DEFAULT_MITRE_LIMIT = 5.0;
+	static const int DEFAULT_ENDCAP_STYLE = ENDCAP_ROUND;
+	static const int DEFAULT_JOIN_STYLE = JOIN_ROUND;
+
+	double mitreLimit = DEFAULT_MITRE_LIMIT;
+	int endCapStyle = DEFAULT_ENDCAP_STYLE;
+	int joinStyle  = DEFAULT_JOIN_STYLE;
+	char *param;
+	char *params = NULL;
+
 
 	PROFSTART(PROF_QRUN);
 
 	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
 	size = PG_GETARG_FLOAT8(1);
-	if ( PG_NARGS() > 2 ) quadsegs = PG_GETARG_INT32(2);
+
+	nargs = PG_NARGS();
 
 	initGEOS(lwnotice, lwnotice);
 
@@ -758,10 +869,141 @@ Datum buffer(PG_FUNCTION_ARGS)
 	g1 = (GEOSGeometry *)POSTGIS2GEOS(geom1);
 	PROFSTOP(PROF_P2G1);
 
+	if (nargs > 2)
+	{
+		/* We strdup `cause we're going to modify it */
+		params = pstrdup(PG_GETARG_CSTRING(2));
+
+		POSTGIS_DEBUGF(3, "Params: %s", params);
+
+		for (param=params; ; param=NULL)
+		{
+			char *key, *val;
+			param = strtok(param, " ");
+			if ( param == NULL ) break;
+			POSTGIS_DEBUGF(3, "Param: %s", param);
+
+			key = param;
+			val = strchr(key, '=');
+			if ( val == NULL || *(val+1) == '\0' )
+			{
+				lwerror("Missing value for buffer "
+				        "parameter %s", key);
+				break;
+			}
+			*val = '\0';
+			++val;
+
+			POSTGIS_DEBUGF(3, "Param: %s : %s", key, val);
+
+			if ( !strcmp(key, "endcap") )
+			{
+				/* Supported end cap styles:
+				 *   "round", "flat", "square"
+				 */
+				if ( !strcmp(val, "round") )
+				{
+					endCapStyle = ENDCAP_ROUND;
+				}
+				else if ( !strcmp(val, "flat") ||
+				          !strcmp(val, "butt")    )
+				{
+					endCapStyle = ENDCAP_FLAT;
+				}
+				else if ( !strcmp(val, "square") )
+				{
+					endCapStyle = ENDCAP_SQUARE;
+				}
+				else
+				{
+					lwerror("Invalid buffer end cap "
+					        "style: %s (accept: "
+					        "'round', 'flat', 'butt' "
+					        "or 'square'"
+					        ")", val);
+					break;
+				}
+
+			}
+			else if ( !strcmp(key, "join") )
+			{
+				if ( !strcmp(val, "round") )
+				{
+					joinStyle = JOIN_ROUND;
+				}
+				else if ( !strcmp(val, "mitre") ||
+				          !strcmp(val, "miter")    )
+				{
+					joinStyle = JOIN_MITRE;
+				}
+				else if ( !strcmp(val, "bevel") )
+				{
+					joinStyle = JOIN_BEVEL;
+				}
+				else
+				{
+					lwerror("Invalid buffer end cap "
+					        "style: %s (accept: "
+					        "'round', 'mitre', 'miter' "
+					        " or 'bevel'"
+					        ")", val);
+					break;
+				}
+			}
+			else if ( !strcmp(key, "mitre_limit") ||
+			          !strcmp(key, "miter_limit")    )
+			{
+				/* mitreLimit is a float */
+				mitreLimit = atof(val);
+			}
+			else if ( !strcmp(key, "quad_segs") )
+			{
+				/* quadrant segments is an int */
+				quadsegs = atoi(val);
+			}
+			else
+			{
+				lwerror("Invalid buffer parameter: %s (accept: "
+				        "'endcap', 'join', 'mitre_limit', "
+				        "'miter_limit and "
+				        "'quad_segs')", key);
+				break;
+			}
+		}
+
+		pfree(params); /* was pstrduped */
+
+		POSTGIS_DEBUGF(3, "endCap:%d joinStyle:%d mitreLimit:%g",
+		               endCapStyle, joinStyle, mitreLimit);
+
+	}
+
+#if POSTGIS_GEOS_VERSION >= 32
+
+	PROFSTART(PROF_GRUN);
+	g3 = GEOSBufferWithStyle(g1, size, quadsegs,
+	                         endCapStyle, joinStyle, mitreLimit);
+	PROFSTOP(PROF_GRUN);
+
+#else /* POSTGIS_GEOS_VERSION < 32 */
+
+	if ( mitreLimit != DEFAULT_MITRE_LIMIT ||
+	        endCapStyle != DEFAULT_ENDCAP_STYLE ||
+	        joinStyle != DEFAULT_JOIN_STYLE )
+	{
+		lwerror("The GEOS version this postgis binary "
+		        "was compiled against (%d) doesn't support "
+		        "specifying a mitre limit != %d or styles different "
+		        "from 'round' (needs 3.2 or higher)",
+		        DEFAULT_MITRE_LIMIT, POSTGIS_GEOS_VERSION);
+	}
+
 	PROFSTART(PROF_GRUN);
 	g3 = GEOSBuffer(g1,size,quadsegs);
 	PROFSTOP(PROF_GRUN);
 
+#endif /* POSTGIS_GEOS_VERSION < 32 */
+
 	if (g3 == NULL)
 	{
 		elog(ERROR,"GEOS buffer() threw an error!");
@@ -1165,7 +1407,6 @@ Datum isvalid(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(result);
 }
 
-#if POSTGIS_GEOS_VERSION >= 31
 /*
 ** IsValidReason is only available in the GEOS
 ** C API > version 3.0
@@ -1207,7 +1448,6 @@ Datum isvalidreason(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 
 }
-#endif
 
 /**
  * overlaps(PG_LWGEOM g1,PG_LWGEOM g2)
@@ -1368,7 +1608,8 @@ Datum contains(PG_FUNCTION_ARGS)
 		if ( result == 1 ) /* completely inside */
 		{
 			PG_RETURN_BOOL(TRUE);
-		} else
+		}
+		else
 		{
 			PG_RETURN_BOOL(FALSE);
 		}
@@ -1568,7 +1809,8 @@ Datum covers(PG_FUNCTION_ARGS)
 		if ( result != -1 ) /* not outside */
 		{
 			PG_RETURN_BOOL(TRUE);
-		} else
+		}
+		else
 		{
 			PG_RETURN_BOOL(FALSE);
 		}
@@ -1699,7 +1941,8 @@ Datum within(PG_FUNCTION_ARGS)
 		if ( result == 1 ) /* completely inside */
 		{
 			PG_RETURN_BOOL(TRUE);
-		} else
+		}
+		else
 		{
 			PG_RETURN_BOOL(FALSE);
 		}
@@ -1830,7 +2073,8 @@ Datum coveredby(PG_FUNCTION_ARGS)
 		if ( result != -1 ) /* not outside */
 		{
 			PG_RETURN_BOOL(TRUE);
-		} else
+		}
+		else
 		{
 			PG_RETURN_BOOL(FALSE);
 		}
@@ -2034,7 +2278,8 @@ Datum intersects(PG_FUNCTION_ARGS)
 		if ( result != -1 ) /* not outside */
 		{
 			PG_RETURN_BOOL(TRUE);
-		} else
+		}
+		else
 		{
 			PG_RETURN_BOOL(FALSE);
 		}
