@@ -21,8 +21,8 @@
 #include <errno.h>
 
 
-/*
- * implementation GiST support and basic LWGEOM operations (like &&)
+/**
+ * @file implementation GiST support and basic LWGEOM operations (like &&)
  */
 
 
@@ -37,6 +37,7 @@ Datum LWGEOM_below(PG_FUNCTION_ARGS);
 Datum LWGEOM_above(PG_FUNCTION_ARGS);
 Datum LWGEOM_overabove(PG_FUNCTION_ARGS);
 Datum LWGEOM_contained(PG_FUNCTION_ARGS);
+Datum LWGEOM_samebox(PG_FUNCTION_ARGS);
 Datum LWGEOM_contain(PG_FUNCTION_ARGS);
 Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS);
 Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS);
@@ -60,7 +61,7 @@ static int counter_leaf = 0;
 static int counter_intern = 0;
 
 
-/* GiST strategies (modified from src/include/access/rtree.h) */
+/** GiST strategies (modified from src/include/access/rtree.h) */
 #define RTLeftStrategyNumber			1
 #define RTOverLeftStrategyNumber		2
 #define RTOverlapStrategyNumber			3
@@ -75,7 +76,7 @@ static int counter_intern = 0;
 #define RTOverAboveStrategyNumber		12
 
 
-/*
+/**
  * all the lwgeom_<same,overlpa,overleft,left,right,overright,overbelow,below,above,overabove,contained,contain>
  *  work the same.
  *  1. get lwgeom1
@@ -374,6 +375,34 @@ Datum LWGEOM_overabove(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(result);
 }
 
+PG_FUNCTION_INFO_V1(LWGEOM_samebox);
+Datum LWGEOM_samebox(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM *lwgeom1 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	PG_LWGEOM *lwgeom2 = (PG_LWGEOM *)PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	bool result;
+	BOX2DFLOAT4 box1;
+	BOX2DFLOAT4 box2;
+
+	POSTGIS_DEBUG(2, "GIST: LWGEOM_samebox --entry");
+
+	errorIfSRIDMismatch(pglwgeom_getSRID(lwgeom1), pglwgeom_getSRID(lwgeom2));
+
+	if ( ! (getbox2d_p(SERIALIZED_FORM(lwgeom1), &box1) && getbox2d_p(SERIALIZED_FORM(lwgeom2), &box2)) )
+	{
+		PG_FREE_IF_COPY(lwgeom1, 0);
+		PG_FREE_IF_COPY(lwgeom2, 1);
+		PG_RETURN_BOOL(FALSE);
+	}
+
+	result = DatumGetBool(DirectFunctionCall2(BOX2D_same,
+	                      PointerGetDatum(&box1), PointerGetDatum(&box2)));
+
+	PG_FREE_IF_COPY(lwgeom1, 0);
+	PG_FREE_IF_COPY(lwgeom2, 1);
+
+	PG_RETURN_BOOL(result);
+}
 
 PG_FUNCTION_INFO_V1(LWGEOM_contained);
 Datum LWGEOM_contained(PG_FUNCTION_ARGS)
@@ -503,17 +532,10 @@ Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 			if (in != (PG_LWGEOM*)DatumGetPointer(entry->key))
 				pfree(in);  /* PG_FREE_IF_COPY */
 
-#if POSTGIS_PGSQL_VERSION >= 82
 			gistentryinit(*retval, PointerGetDatum(rr),
 			              entry->rel, entry->page,
 			              entry->offset,
 			              FALSE);
-#else
-			gistentryinit(*retval, PointerGetDatum(rr),
-			              entry->rel, entry->page,
-			              entry->offset, sizeof(BOX2DFLOAT4),
-			              FALSE);
-#endif
 
 
 		}
@@ -521,13 +543,8 @@ Datum LWGEOM_gist_compress(PG_FUNCTION_ARGS)
 		{
 			POSTGIS_DEBUG(4, "GIST: LWGEOM_gist_compress got a NULL key");
 
-#if POSTGIS_PGSQL_VERSION >= 82
 			gistentryinit(*retval, (Datum) 0, entry->rel,
 			              entry->page, entry->offset, FALSE);
-#else
-			gistentryinit(*retval, (Datum) 0, entry->rel,
-			              entry->page, entry->offset, 0, FALSE);
-#endif
 
 		}
 
@@ -559,9 +576,10 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 	bool *recheck = (bool *) PG_GETARG_POINTER(4);
 
 	/* We set recheck to false to avoid repeatedly pulling every "possibly matched" geometry
-	   out during index scans. For cases when the geometries are large, doing this
+	   out during index scans. For cases when the geometries are large, rechecking
 	   can make things twice as slow. */
 	*recheck = false;
+
 #endif
 
 	POSTGIS_DEBUG(2, "GIST: LWGEOM_gist_consistent called");
@@ -573,7 +591,7 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 	}
 
 	/*
-	** First pull only a small amount of the tuple, enough to 
+	** First pull only a small amount of the tuple, enough to
 	** get the bounding box, if one exists.
 	** size = header + type + box2df4?
 	*/
@@ -588,7 +606,7 @@ Datum LWGEOM_gist_consistent(PG_FUNCTION_ARGS)
 
 	/*
 	** If the bounding box exists, copy it into the working variable.
-	** If not, pull the full toasted data out, and call the standard box 
+	** If not, pull the full toasted data out, and call the standard box
 	** retrieval function, which will calculate the box from scratch.
 	*/
 	serialized_lwgeom = SERIALIZED_FORM(query);
@@ -648,22 +666,9 @@ lwgeom_rtree_internal_consistent(BOX2DFLOAT4 *key, BOX2DFLOAT4 *query,
 		           (query->ymin<= key->ymax)));
 
 
-#if POSTGIS_DEBUG_LEVEL >=4
-		/*keep track and report info about how many times this is called */
-		if (counter_intern == 0)
-		{
-			POSTGIS_DEBUGF(4, "search bounding box is: <%.16g %.16g,%.16g %.16g> - size box2d= %ld",
-			               query->xmin,query->ymin,query->xmax,query->ymax,sizeof(BOX2DFLOAT4));
-
-		}
-
-
-		POSTGIS_DEBUGF(4, "%i:(int)<%.8g %.8g,%.8g %.8g>&&<%.8g %.8g,%.8g %.8g> %i",counter_intern,key->xmin,key->ymin,key->xmax,key->ymax,
+		POSTGIS_DEBUGF(4, "%i:(int)<%.8g %.8g,%.8g %.8g>&&<%.8g %.8g,%.8g %.8g> %i",counter_intern++,key->xmin,key->ymin,key->xmax,key->ymax,
 		               query->xmin,query->ymin,query->xmax,query->ymax,   (int) retval);
 
-		counter_intern++;
-#endif
-
 		return(retval);
 		break;
 
@@ -726,13 +731,11 @@ lwgeom_rtree_leaf_consistent(BOX2DFLOAT4 *key,
 		          ((query->ymax>= key->ymax) &&
 		           (query->ymin<= key->ymax)));
 
-#if POSTGIS_DEBUG_LEVEL >= 4
 		/*keep track and report info about how many times this is called */
 		POSTGIS_DEBUGF(4, "%i:gist test (leaf) <%.6g %.6g,%.6g %.6g> &&  <%.6g %.6g,%.6g %.6g> --> %i",
-		               counter_leaf,key->xmin,key->ymin,key->xmax,key->ymax,
+		               counter_leaf++,key->xmin,key->ymin,key->xmax,key->ymax,
 		               query->xmin,query->ymin,query->xmax,query->ymax,   (int) retval);
-		counter_leaf++;
-#endif
+
 		return(retval);
 
 		break;
@@ -785,7 +788,7 @@ Datum LWGEOM_gist_decompress(PG_FUNCTION_ARGS)
 
 
 
-/*
+/**
  * The GiST Union method for boxes
  * returns the minimal bounding box that encloses all the entries in entryvec
  */
@@ -833,7 +836,7 @@ Datum LWGEOM_gist_union(PG_FUNCTION_ARGS)
 }
 
 
-/*
+/**
  * size of a box is width*height
  * we do this in double precision because width and height can be very very small
  * and it gives screwy results
@@ -865,7 +868,7 @@ size_box2d(Datum box2d)
 	return result;
 }
 
-/*
+/**
  * size of a box is width*height
  * we do this in double precision because width and height can be very very small
  * and it gives screwy results
@@ -903,7 +906,7 @@ static double size_box2d_double(Datum box2d)
 
 
 
-/*
+/**
 ** The GiST Penalty method for boxes
 ** As in the R-tree paper, we use change in area as our penalty metric
 */
@@ -959,7 +962,7 @@ compare_KB(const void* a, const void* b)
 	return ( sa>sb ) ? 1 : -1;
 }
 
-/*
+/**
 ** Equality method
 */
 PG_FUNCTION_INFO_V1(LWGEOM_gist_same);
@@ -980,7 +983,7 @@ Datum LWGEOM_gist_same(PG_FUNCTION_ARGS)
 
 
 
-/*
+/**
 ** The GiST PickSplit method
 ** New linear algorithm, see 'New Linear Node Splitting Algorithm for R-tree',
 ** C.H.Ang and T.C.Tan
@@ -1174,7 +1177,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 		                                    PointerGetDatum(unionB), PointerGetDatum(unionT));
 		float sizeLR, sizeBT;
 
-		/*elog(NOTICE,"direction is abigeous"); */
+		/*elog(NOTICE,"direction is ambiguous"); */
 
 		sizeLR = size_box2d(interLR);
 		sizeBT = size_box2d(interBT);
@@ -1213,7 +1216,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 			POSTGIS_DEBUGF(4, "   posL = %i, posR=%i", posL,posR);
 			POSTGIS_DEBUG(4, "   posL's (nleft) offset numbers:");
 
-			for (i=0;i<posL;i++)
+			for (i=0; i<posL; i++)
 			{
 				sprintf(bbb," %i", listL[i]);
 				strcat(aaa,bbb);
@@ -1221,7 +1224,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 			POSTGIS_DEBUGF(4, "%s", aaa);
 			aaa[0]=0;
 			POSTGIS_DEBUG(4, "   posR's (nright) offset numbers:");
-			for (i=0;i<posR;i++)
+			for (i=0; i<posR; i++)
 			{
 				sprintf(bbb," %i", listR[i]);
 				strcat(aaa,bbb);
@@ -1255,7 +1258,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 			POSTGIS_DEBUGF(4, "   posB = %i, posT=%i", posB,posT);
 			POSTGIS_DEBUG(4, "   posB's (nleft) offset numbers:");
 
-			for (i=0;i<posB;i++)
+			for (i=0; i<posB; i++)
 			{
 				sprintf(bbb," %i", listB[i]);
 				strcat(aaa,bbb);
@@ -1263,7 +1266,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 			POSTGIS_DEBUGF(4, "%s", aaa);
 			aaa[0]=0;
 			POSTGIS_DEBUG(4, "   posT's (nright) offset numbers:");
-			for (i=0;i<posT;i++)
+			for (i=0; i<posT; i++)
 			{
 				sprintf(bbb," %i", listT[i]);
 				strcat(aaa,bbb);
@@ -1278,7 +1281,7 @@ Datum LWGEOM_gist_picksplit(PG_FUNCTION_ARGS)
 }
 
 
-/* debug function */
+/** debug function */
 Datum report_lwgeom_gist_activity(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(report_lwgeom_gist_activity);
 Datum report_lwgeom_gist_activity(PG_FUNCTION_ARGS)
