@@ -7,10 +7,10 @@
 
 CREATE INDEX quick_gist on test using gist (the_geom gist_geometry_ops);
 
- select num,astext(the_geom) from test where the_geom && 'BOX3D(125 125,135 135)'::box3d order by num;
+ select num,ST_astext(the_geom) from test where the_geom && 'BOX3D(125 125,135 135)'::box3d order by num;
 
 set enable_seqscan = off;
 
- select num,astext(the_geom) from test where the_geom && 'BOX3D(125 125,135 135)'::box3d  order by num;
+ select num,ST_astext(the_geom) from test where the_geom && 'BOX3D(125 125,135 135)'::box3d  order by num;
 
 DROP TABLE test;
