@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# $Id: test_estimation.pl,v 1.9 2005/04/18 13:30:25 strk Exp $
+# $Id: test_estimation.pl 4652 2009-10-15 17:44:48Z mcayland $
 # 
 # TODO:
 #
@@ -263,7 +263,7 @@ sub test_extent
 	local($ext) = shift;
 
 	# Test whole extent query
-	$query = 'explain analyze select oid from "'.
+	$query = 'explain analyze select 1 from "'.
 		$SCHEMA.'"."'.$TABLE.'" WHERE "'.$COLUMN.'" && '.
 		"setSRID('BOX3D(".$ext->{'xmin'}." ".
 		$ext->{'ymin'}.", ".$ext->{'xmax'}." ".
@@ -285,7 +285,7 @@ sub test_extent
 }
 
 # 
-# $Log: test_estimation.pl,v $
+# $Log$
 # Revision 1.9  2005/04/18 13:30:25  strk
 # Fixed to work against LWGEOM installations
 #
